#include "em1.h"

/************************************************************************/
/* irand(iseed) returns values 1 through 2**31-2.                       */
/* From "Random number generators: good ones are hard to find", S. Park */
/* and K. Miller, Communications of ACM, October 1988, pp 1192-1201.    */
/* This is from page 1195, and is to work on any system for which       */
/* maxint is 2**31-1 or larger. Due earlier to Schrage, as cited by P&M.*/
/*                                                                      */
/* Note: OK values for iseed are 1 through 2**31-2. Give it 0 or 2*31-1 */
/* and it will return the same values thereafter!                       */
/*                                                                      */
/* C version 6/91, Bruce Langdon.                                       */
/*                                                                      */
/* Algorithm replaces seed by mod(a*seed,m). First represent            */
/* seed = q*hi + lo.  Then                                              */
/* a*seed = a*q*hi + lo = (m - r)*hi + a*lo = (a*lo - r*hi) + m*hi,     */
/* and new seed = a*lo - r*hi unless negative; if so, then add m.       */

SCALAR frand()
{
  long a = 16807, m = 2147483647, q = 127773, r = 2836;
  long hi, lo;
  static long seed=31207321;
  SCALAR fnumb;

  hi = seed/q;
  lo = seed - q*hi;
  seed = a*lo - r*hi;
  /* "seed" will always be a legal integer of 32 bits (including sign). */
  if(seed <= 0) seed = seed + m;
  fnumb = seed/2147483646.0;

  return(fnumb);
}

/***********************************************************************/
/* Subroutine CREATOR loads particles in phase space.  Both a 3-D
   isotropic Maxwellian thermal spread and a drift in Vx may be
   specified.  The particles are loaded uniformly in space with a
   possible perturbation given by Xs1 and MODE.	 */

Creator(isp,mode)
int isp, mode;
{
  int i, n=128, iq;
  SCALAR qm = -1.0, wp=1.0, temp=0.0, v0=0.0, xperturb=0.0, freq=0.1,
  pmax, dp, pp, fv, gamma_func0, emc2, gamtemp, ddx, df;
  char a_char[80];

  if (InputDeck)
    {
#ifndef XG_SCALAR_DOUBLE
        while (fscanf(InputDeck, "%d %g %g %g", 
#else
            while (fscanf(InputDeck, "%d %lg %lg %lg", 
#endif
          &n, &qm, &wp, &temp) <4)
	fscanf(InputDeck, "%s", a_char);
#ifndef XG_SCALAR_DOUBLE
while (fscanf(InputDeck, "%g %g %g %g", 
#else
    while (fscanf(InputDeck, "%lg %lg %lg %lg", 
#endif
          &v0, &xperturb) <2)
	fscanf(InputDeck, "%s", a_char);
    }

  /***********************Check Input Values***************************/
  if (!qm)
    {
      printf("\nqm[%d] cannot equal 0!\n",isp);
      exit(1);
    }
  if (!wp)
    {
      printf("\nwp[%d] cannot equal 0!\n",isp);
      exit(1);
    }
  /********************************************************************/
  printf("\nnp[%d]=%d\tqm[%d]=%g\twp[%d]=%g\ttemp[%d]=%g\n",isp,n,isp,qm,
	 isp,wp,isp,temp);
  printf("v0[%d]=%g\t xperturb[%d]=%g\n",isp,v0,isp,xperturb);
  /********************************************************************/
  np[isp]= n;
  qs[isp]= wp*wp/(n*qm);
  ms[isp]= qs[isp]/qm;
  nm[isp]= n*(ms[isp]);

  if (temp)
    {
      pmax = 5.0*sqrt(temp);
      dp = 2.0*pmax/n;
      pos[isp][0] = 0.0;

      /*Save the indefinite integral of the CDF in the x-array*/
      for (i=1; i<n; i++)
	{
	  pp = (i-.5)*dp - pmax;
	  fv = exp(-pp*pp/(2.0*temp));
	  pos[isp][i] = pos[isp][i-1] + max(fv,0.0);
	}
      /*Load particles in momentum space according to the CDF.
	each particle is loaded with px = py = pz.	*/
      df = pos[isp][n-1]/n;
      iq = 0;
      for (i=0; i<n; i++)
	{
	  fv = (i+0.5)*df;
	  while (fv>=pos[isp][iq+1])
	    {
	      iq++;
	      if (iq>(n-2))
		{
		  printf("Distribution Function Error");
		  exit(1);
		}
	    }
	  vx[isp][i]=vy[isp][i]=vz[isp][i] =
	    dp*(iq+(fv-pos[isp][iq])/(pos[isp][iq+1]-pos[isp][iq]))-pmax;
	}
    }
  /*Load particles in configuration space uniformly with
    perturbation if need be.	*/
  xperturb *= length/n;
  ddx = length/n;
  for (i=0; i<n; i++)
    {
      pos[isp][i] = (i+0.5)*ddx;
      pos[isp][i] += xperturb*cos(TWOPI*mode*pos[isp][i]/length);
    }

  /*Randomize the particle momenta and positions so as
    to decorrelate phase space.	*/
  Smear(isp);

  /*Calculate velocities from momenta.*/
  gamma_func0 = sqrt(1.0 - v0*v0);
  for (i=0; i<n; i++)
    {
      emc2 = sqrt(1.0+vx[isp][i]*vx[isp][i]+vy[isp][i]*vy[isp][i]+
		  vz[isp][i]*vz[isp][i]);
      vx[isp][i] = (vx[isp][i] + v0*emc2)/gamma_func0;
      vy[isp][i] /= gamma_func0;
      vz[isp][i] /= gamma_func0;
      gamtemp = sqrt(1.0+vx[isp][i]*vx[isp][i]+vy[isp][i]*vy[isp][i]+
		     vz[isp][i]*vz[isp][i]);
      vx[isp][i] /= gamtemp;
      vy[isp][i] /= gamtemp;
      vz[isp][i] /= gamtemp;
    }
}  

/**************SMEAR randomizes vector arrays by random pair exchanges.****/

Smear(isp)
int isp;
{
  int i,ii;
  SCALAR xx,vxx,vyx,vzx,num;

  for (i=0; i<np[isp]; i++)
    {
      ii		= np[isp]*frand();
      xx		= pos[isp][i];
      pos[isp][i]	= pos[isp][ii];
      pos[isp][ii]= xx;
      ii		= np[isp]*frand();
      vxx		= vx[isp][i];
      vx[isp][i]	= vx[isp][ii];
      vx[isp][ii]	= vxx;
      ii		= np[isp]*frand();
      vyx		= vy[isp][i];
      vy[isp][i]	= vy[isp][ii];
      vy[isp][ii]	= vyx;
      ii		= np[isp]*frand();
      vzx		= vz[isp][i];
      vz[isp][i]	= vz[isp][ii];
      vz[isp][ii]	= vzx;
    }
}
/*****************************End of SMEAR********************************/


