/* MOVER advances the particle velocities and positions.
   Charge densities and currents are also accumulated, as well as
   calculating the relativistic temperature of the species. */

#include "em1.h"

Mover(isp)
int isp;
{
  SCALAR	abx,qdx,qdt,ae,del,aexpt,aeypt,aezpt,abxpt,abypt,abzpt,
        gvxs,gvys,gvzs,vvx,vvy,vvz,f,delrs,dells,
        dcg,reltemp,thermt,gosci,pxtherm,pytherm,pztherm,
        pxoscsq,pyoscsq,pzoscsq,relts;
	
  int	i, ij, n[MAXG1];
  
  /* Set various constants */
  if (!isp) ke = 0.0;
  abx = tx[isp];
  qdx = qs[isp]/dx;
  qdt = qs[isp]/dt;
  ae = 0.5*(qs[isp]/ms[isp])*dt*dt/dx;
  
  /* Renormalize the fields and zero arrays */
  for (i=0; i<ng; i++)
    {
      n[i] = 0.0;
      vxosc[i] = vyosc[i] = vzosc[i] = 0.0;
      thermx[i] = 0.0;
    }
  if (!isp)
    {
      for (i=0; i<ng; i++)
	{
	  rho[i] = rho0;
	  jym[i] = jyp[i] = jzm[i] = jzp[i] = 0.0;
	}
    }
  rho[nc] = 0.0;
  for (i=0; i<np[isp]; i++)
    {
      
      /*Interpolate fields to particle position*/
      ij	= pos[isp][i];
      del	= pos[isp][i]-ij;
      aexpt = ae*(ex[ij] + del*(ex[ij+1]-ex[ij]));
      aeypt = ae*(ey[ij] + del*(ey[ij+1]-ey[ij]));
      aezpt = ae*(ez[ij] + del*(ez[ij+1]-ez[ij]));
      abypt = ae*(by[ij] + del*(by[ij+1]-by[ij]));
      abzpt = ae*(bz[ij] + del*(bz[ij+1]-bz[ij])) + tz[isp];
      
      /*Half-acceleration along E.*/
      gvxs	= vx[isp][i]*gamma_func[isp][i]+aexpt;
      gvys	= vy[isp][i]*gamma_func[isp][i]+aeypt;
      gvzs	= vz[isp][i]*gamma_func[isp][i]+aezpt;
      
      /*First half-rotation, using v x B /gamma_func.*/
      gamma_func[isp][i] = sqrt(1.0+gvxs*gvxs + gvys*gvys + gvzs*gvzs);
      ke		+= (gamma_func[isp][i]-1.0)*ms[isp];
      dcg	= dt/(dx*gamma_func[isp][i]);
      abxpt	= dcg*abx;
      abypt	*= dcg;
      abzpt *= dcg;
      vvx	= gvxs + gvys*abzpt - gvzs*abypt;
      vvy	= gvys + gvzs*abxpt - gvxs*abzpt;
      vvz	= gvzs + gvxs*abypt - gvys*abxpt;
      
      /*Second half-rotation and second half-acceleration*/
      f		= 2.0/(1.0+abxpt*abxpt + abypt*abypt + abzpt*abzpt);
      abxpt *= f;
      abypt *= f;
      abzpt *= f;
      gvxs	+= vvy*abzpt - vvz*abypt + aexpt;
      gvys	+= vvz*abxpt - vvx*abzpt + aeypt;
      gvzs	+= vvx*abypt - vvy*abxpt + aezpt;
      
      /*Use v = p/(mc*gamma_func) */
      gamma_func[isp][i]	= sqrt(1.0+gvxs*gvxs + gvys*gvys + gvzs*gvzs);
      vx[isp][i] = gvxs/gamma_func[isp][i];
      vy[isp][i] = gvys/gamma_func[isp][i];
      vz[isp][i] = gvzs/gamma_func[isp][i];
      px[isp]	+= gvxs;		
      py[isp]	+= gvys;
      pz[isp]	+= gvzs;		
      
      /*Compute old particle position quantities*/
      ij		= pos[isp][i];
      delrs	= pos[isp][i]-ij;
      dells	= 1.0-delrs;
      jym[ij]	+= dells*qdt*vy[isp][i];
      jym[ij+1]+= delrs*qdt*vy[isp][i];
      jzm[ij]	+= dells*qdt*vz[isp][i];
      jzm[ij+1]+= delrs*qdt*vz[isp][i];
      
      /*Compute new particle position quantities*/
      pos[isp][i]	+= vx[isp][i];
      if (pos[isp][i] < 0.0) pos[isp][i] += nc;
      if (pos[isp][i] >= nc) pos[isp][i] -= nc;
      ij		= pos[isp][i];
      delrs = pos[isp][i]-ij;
      dells = 1.0-delrs;
      rho[ij]	+= dells*qdx;
      rho[ij+1]+= delrs*qdx;
      jyp[ij]	+= dells*qdt*vy[isp][i];
      jyp[ij+1]+= delrs*qdt*vy[isp][i];
      jzp[ij]	+= dells*qdt*vz[isp][i];
      jzp[ij+1]+= delrs*qdt*vz[isp][i];
      n[ij]		+= 1;
      vxosc[ij]+= vx[isp][i];
      vyosc[ij]+= vy[isp][i];
      vzosc[ij]+= vz[isp][i];
    }
  rho[0]	+= rho[nc];
  jym[0]	+= jym[nc];
  jyp[0]	+= jyp[nc];
  jzm[0]	+= jzm[nc];
  jzp[0]	+= jzp[nc];
  n[0]		+= n[nc];
  vxosc[0] += vxosc[nc];
  vyosc[0] += vyosc[nc];
  vzosc[0] += vzosc[nc];
  rho[nc]	= rho[0];
  jym[nc]	= jym[0];
  jyp[nc]	= jyp[0];
  jzm[nc]	= jzm[0];
  jzp[nc]	= jzp[0];
  n[nc]		= n[0];
  vxosc[nc]= vxosc[0];
  vyosc[nc]= vyosc[0];
  vzosc[nc]= vzosc[0];
  px[isp]	/= np[isp];  
  py[isp]	/= np[isp];
  pz[isp]	/= np[isp];

  for (i=0; i<ng; i++)
    {
      if(n[i])
	{
	  vxosc[i] /= n[i];
	  vyosc[i] /= n[i];
	  vzosc[i] /= n[i];
	}
    }

  reltemp = 0.0;
  for (i=0; i<np[isp]; i++)
    {
      ij = pos[isp][i];
      gosci = 1.0/sqrt(1.0-vxosc[ij]*vxosc[ij] - vyosc[ij]*vyosc[ij]
		       - vzosc[ij]*vzosc[ij]);
      pxtherm = vx[isp][i]*gamma_func[isp][i];
      pytherm = vy[isp][i]*gamma_func[isp][i];
      pztherm = vz[isp][i]*gamma_func[isp][i];
      pxoscsq = gosci*vxosc[ij]*gosci*vxosc[ij];
      pyoscsq = gosci*vyosc[ij]*gosci*vyosc[ij];
      pzoscsq = gosci*vzosc[ij]*gosci*vzosc[ij];
      relts	= 0.5110041e6*(sqrt(1.0+pxtherm*pxtherm + pytherm*pytherm +
				    pztherm*pztherm) - sqrt(1.0+pxoscsq+pyoscsq+pzoscsq));
      reltemp += relts;
      thermx[ij] += relts/n[ij];
    }
  thermt = reltemp/np[isp];
  thermx[0] += thermx[nc];
  thermx[nc] = thermx[0];
}
